#!/usr/bin/perl -w -I/usr/sausalito/perl

use strict;

use CCE;
use Sauce::Util;
use Sauce::Validators;

#declares that should probably go elsewhere:
my $confBlockStart = "# Veritas NetBackup Config Start"; # Comment block for config files
my $confBlockStop = "# Veritas NetBackup Config Stop";

my $confFile	     = "/opt/openv/netbackup/bp.conf";
my $inetdFile        = "/etc/inetd.conf";
my $servFile         = "/etc/services";
my $inetdPidFile     = "/var/run/inetd.pid";


my $cce = new CCE ( Namespace => "NetBackup" );

$cce->connectfd();

# Need to test if event_object successful... what do we look at?
my $netbackup = $cce->event_object();
my $vtNetBackupMaster =  $netbackup->{vtNetBackupMaster};
my @vtNetBackupExtras = $cce->scalar_to_array($netbackup->{vtNetBackupExtras});

my $enabled = $netbackup->{enable};

if ($enabled && !$vtNetBackupMaster && !@vtNetBackupExtras) {
    $cce->warn("[[veritas-netbackup.enabledButNoData]]");
    $cce->bye("FAIL");
    die;
}

if ($enabled && $vtNetBackupMaster &&  !checkDNSLookup($vtNetBackupMaster)) {
    $cce->warn("[[veritas-netbackup.vtNetBackupMasterField_LookupFailed]]");
    $cce->bye("FAIL");
    die;
}       
my $confData = "SERVER = $vtNetBackupMaster\n";

foreach my $server (@vtNetBackupExtras) {
    if ($enabled && !checkDNSLookup($server)) {
	$cce->warn("[[veritas-netbackup.vtNetBackupExtrasField_LookupFailed]]");
	$cce->bye("FAIL");
	die;
    }
    $confData .= "SERVER = $server\n";
}

my $hostname = `hostname`;
chomp($hostname);
if ($enabled && !checkDNSLookup($hostname)) {
    $cce->warn("[[veritas-netbackup.vtNetBackupLocalHost_LookupFailed]]");
    $cce->bye("FAIL");
    die;
}
$confData .= "CLIENT_NAME = $hostname\n";


if(! -e $confFile){
    system("/bin/mkdir",'-p',"/opt/openv/netbackup");
}

 Sauce::Util::replaceblock($confFile, $confBlockStart, $confData,$confBlockStop,0644);

# Start service

if ($enabled) {
    my $inetData .= "bpcd    stream  tcp     nowait  root    /usr/openv/netbackup/bin/bpcd   bpcd\n";
    $inetData .= "vopied  stream  tcp     nowait  root    /usr/openv/bin/vopied   vopied\n";
    $inetData .= "bpjava-msvc     stream  tcp     nowait  root    /usr/openv/netbackup/bin/bpjava-msvc -transient\n";
  Sauce::Util::replaceblock($inetdFile, $confBlockStart, $inetData,$confBlockStop,0644);
    
    my $servData .= "bprd    13720/tcp       bprd\n";
    $servData .= "bpcd    13782/tcp       bpcd\n";
    $servData .= "vopied  12783/tcp       vopied\n";
    $servData .= "bpjava-msvc     13722/tcp       bpjava-msvc\n";
    
  Sauce::Util::replaceblock($servFile, $confBlockStart, $servData,$confBlockStop,0644);
    
}else{
    my $inetData = "";
  Sauce::Util::replaceblock($inetdFile, $confBlockStart, $inetData,$confBlockStop,0644);
    
    my $servData = "";
  Sauce::Util::replaceblock($servFile, $confBlockStart, $servData,$confBlockStop,0644);
}

# Now, HUP Inetd to make the changes take effect.
if ( -f $inetdPidFile) {
    system("kill -HUP `cat $inetdPidFile`");
} else {
    system("killall -HUP inetd");
}

$cce->bye("SUCCESS");

# The basic algorithm is this:
# hostname => ip addresses
# for each address, ipAddress => hostname2 => ipAddress2
# make sure ipAddress == ipAddress2. If they are all equal, good. If not, bad.
# Here are 2 examples:
# --------------------
# www.yahoo.com => 204.71.202.160, 216.115.105.2, 204.71.200.74,
#                  204.71.200.75, 204.71.200.67, 204.71.200.68
# 204.71.202.160 => w1.yahoo.com => 204.71.202.160
#
# They match. Good.
# --------------------
# yahoo.com => 204.71.200.243, 204.71.200.245
# 204.71.200.243 => UNKNOWN.yahoo.com => does not exist
# 204.71.200.245 => UNKNOWN.yahoo.com => does not exist
# They don't exist. Bad
sub checkDNSLookup {
    use Socket;
    my $host = shift; 
    my ($name,$aliases,$addrtype,$length,@addrs) = gethostbyname($host) or return 0;
    my $ipaddr;
    foreach $ipaddr (@addrs) {
	my $revHost = gethostbyaddr($ipaddr, AF_INET) or return 0;
	my $revHostToIP =  gethostbyname($revHost) or return 0;
	if ($ipaddr != $revHostToIP) {
	    return 0;
	}
    }
    return 1;
}
